/*
matmax.c

reads in an m x n float matrix and writes out row and column number 
(array index) of the largest element.
version 0.03
Peter A. Cejchan, GPL
Tue Mar 23 16:17:05 EST 2004 <cej@gli.cas.cz | pac7@post.cz | pac@plan9.cz>


*/


#include <u.h>
#include <libc.h>
#include <bio.h>
#include <ctype.h>

#define MAXROWS 100
#define MAXCOLS MAXROWS
#define MAXLENGTH MAXROWS
#define MAXFLOAT 1e20
typedef struct Matrix Matrix;
typedef struct iMatrix iMatrix;
typedef struct Vector Vector;
typedef struct iVector iVector;

struct Matrix{
	int rows, cols;
	double x[MAXROWS][MAXCOLS];
};

struct iMatrix{
	int rows, cols;
	int x[MAXROWS][MAXCOLS];
};

struct Vector{
	int length;
	double x[MAXLENGTH];
};

struct iVector{
	int length;
	int x[MAXLENGTH];
};


int 
readmat(Matrix *mat){
	Biobuf bin;
	char *buf, *av[128];
	int i, j;

	Binit(&bin, 0, OREAD);
	i=0;
	while((buf = Brdstr(&bin, '\n', 1)) != nil){
		mat->cols=   tokenize(buf, av, MAXCOLS);
		for(j=0; j<mat->cols; j++){
			mat->x[i][j]=atof(av[j]);
		}
		free(buf);
		i++;
	}
	mat->rows=i;
	return 1;
}

int 
matmax(Matrix *mat){
	int i, j, maxi, maxj;
	double maxx;
	maxx = -MAXFLOAT;
	for(i=0; i<mat->rows; i++){
		for(j=0; j<mat->cols; j++){
//			print("%f ", mat->x[i][j]);
			if(mat->x[i][j] > maxx) {
				maxx = mat->x[i][j];
				maxi = i; maxj = j;
			}
		}
//		print("\n");
	}
//	print("%f\n", maxx);
	print("%d %d\n", maxi, maxj);
	return 1;
}

int 
mattranspose(Matrix *mat){
	Matrix tmp;
	int i, j;

	for(i=0; i<mat->rows; i++){
		for(j=0; j<mat->cols; j++){
			tmp.x[j][i] = mat->x[i][j];
		}
	}
	for(i=0; i<mat->rows; i++){
		for(j=0; j<mat->cols; j++){
			mat->x[i][j] = tmp.x[i][j];
		}
	}
	return 1;
}

void 
matmul(Matrix *a, Matrix *b){

//untested!!!

	int i, j, k;
	double sum;
	Matrix tmp;
	for(i=0;i<a->rows;i++) for(j=0;j<a->rows;j++){
		sum=0;
		for(k=0;k<a->cols;k++)
			sum+=a->x[i][k]*b->x[k][j];
		tmp.x[i][j]=sum;
	}
	for(i=0;i<a->rows;i++) for(j=0;j<a->rows;j++)
		a->x[i][j]=tmp.x[i][j];
}

int 
readvec(Vector *vec){
	Biobuf bin;
	char *buf, *av[128];
	int i, j;

	Binit(&bin, 0, OREAD);
	i=0;
	if((buf = Brdstr(&bin, '\n', 1)) != nil){
		vec->length=   tokenize(buf, av, MAXLENGTH);
		for(j=0; j<vec->length; j++){
			vec->x[j]=atof(av[j]);
		}
		free(buf);
	}
	return 1;
}

int 
readivec(iVector *vec){
	Biobuf bin;
	char *buf, *av[128];
	int i, j;

	Binit(&bin, 0, OREAD);
	i=0;
	if((buf = Brdstr(&bin, '\n', 1)) != nil){
		vec->length=   tokenize(buf, av, MAXLENGTH);
		for(j=0; j<vec->length; j++){
			vec->x[j]=atoi(av[j]);
		}
		free(buf);
	}
	return 1;
}


int
neighborpair(iVector *vec, iMatrix *mat){

//untested!!!

	int i, j;
	for(i=0; i<mat->rows; i++){
		for(j=0; j<mat->cols; j++){
			mat->x[i][j]=0;
		}
	}
	for(i=0; i<vec->length-1; i++){
		mat->x[vec->x[i]][vec->x[i+1]]=0;
		mat->x[vec->x[i+1]][vec->x[i]]=0;
	}

}


void 
main(void){
	Matrix mat; 

if(!readmat(&mat)) abort;
if(!matmax(&mat)) abort;
}


